@@ -1,25 +1,19 @@
-/* LICENSE BEGIN
-    This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
-
-    SPDX-License-Identifier: GPL-3.0-only
-    This file is also available under commercial licensing terms.
-    Please contact info@sixtyfps.io for more information.
-LICENSE END */
+// Copyright © SixtyFPS GmbH <info@sixtyfps.io>
+// SPDX-License-Identifier: (GPL-3.0-only OR LicenseRef-SixtyFPS-commercial)
 
 //! implementation of vtable::Vrc
 
 use super::*;
+use atomic_polyfill::{AtomicU32, Ordering};
 use core::convert::TryInto;
-use core::sync::atomic::AtomicU32;
-use std::sync::atomic::Ordering;
 
-/// This trait is implemented by the `#[vtable]` macro.
+/// This trait is implemented by the [`#[vtable]`](macro@vtable) macro.
 ///
 /// It is implemented if the macro has a "drop_in_place" function.
 ///
-/// Safety: the implementation of drop_in_place and dealloc must be correct
+/// # Safety
+///
+/// The implementation of drop_in_place and dealloc must be correct
 pub unsafe trait VTableMetaDropInPlace: VTableMeta {
     /// # Safety
     /// The target ptr argument needs to be pointing to a an instance of the VTable
@@ -36,7 +30,7 @@ pub unsafe trait VTableMetaDropInPlace: VTableMeta {
 /// actual type is not known.
 pub struct Dyn(());
 
-/// Similar to [`std::alloc::Layout`], but `repr(C)`
+/// Similar to [`core::alloc::Layout`], but `repr(C)`
 #[repr(C)]
 #[derive(Clone, Copy)]
 pub struct Layout {
@@ -90,12 +84,12 @@ impl<'vt, VTable: VTableMeta, X> VRcInner<'vt, VTable, X> {
 
 /// A reference counted pointer to an object matching the virtual table `T`
 ///
-/// Similar to [`std::rc::Rc`] where the `VTable` type parameter is a VTable struct
-/// annotated with `#[vtable]`, and the `X` type parameter is the actual instance.
+/// Similar to [`alloc::rc::Rc`] where the `VTable` type parameter is a VTable struct
+/// annotated with [`#[vtable]`](macro@vtable), and the `X` type parameter is the actual instance.
 /// When `X` is the [`Dyn`] type marker, this means that the X is not known and the only
 /// thing that can be done is to get a [`VRef<VTable>`] through the [`Self::borrow()`] function.
 ///
-/// Other differences with the `std::rc::Rc` types are:
+/// Other differences with the [`alloc::rc::Rc`] types are:
 /// - It does not allow to access mutable reference. (No `get_mut` or `make_mut`), meaning it is
 /// safe to get a Pin reference with `borrow_pin`.
 /// - It is safe to pass it across ffi boundaries.
@@ -148,7 +142,7 @@ impl<VTable: VTableMetaDropInPlace, X: HasStaticVTable<VTable>> VRc<VTable, X> {
             layout.align().max(layout_with_layout.align()),
         )
         .unwrap();
-        let mem = unsafe { std::alloc::alloc(layout) as *mut VRcInner<VTable, X> };
+        let mem = unsafe { alloc::alloc::alloc(layout) as *mut VRcInner<VTable, X> };
         let inner = NonNull::new(mem).unwrap();
         assert!(!mem.is_null());
 
@@ -177,6 +171,22 @@ impl<VTable: VTableMetaDropInPlace, X: HasStaticVTable<VTable>> VRc<VTable, X> {
     }
 }
 
+impl<VTable: VTableMetaDropInPlace + 'static, X: HasStaticVTable<VTable> + 'static> VRc<VTable, X> {
+    /// This function allows safely holding a reference to a field inside the VRc. In order to accomplish
+    /// that, you need to provide a mapping function `map_fn` in which you need to provide and return a
+    /// pinned reference to the object you would like to map. The returned `VRcMapped` allows obtaining
+    /// that pinned reference again using [`VRcMapped::as_pin_ref`].
+    pub fn map<MappedType: ?Sized>(
+        this: Self,
+        map_fn: impl for<'r> FnOnce(Pin<&'r X>) -> Pin<&'r MappedType>,
+    ) -> VRcMapped<VTable, MappedType> {
+        VRcMapped {
+            parent_strong: Self::into_dyn(this.clone()),
+            object: map_fn(this.as_pin_ref()).get_ref(),
+        }
+    }
+}
+
 impl<VTable: VTableMetaDropInPlace, X> VRc<VTable, X> {
     /// Create a Pinned reference to the inner.
     ///
@@ -245,7 +255,7 @@ unsafe impl<VTable: VTableMetaDropInPlace + 'static, X: Send + Sync> Sync for VR
 /// Weak pointer for the [`VRc`] where `VTable` is a VTable struct, and
 /// `X` is the type of the instance, or [`Dyn`] if it is not known
 ///
-/// Similar to [`std::rc::Weak`].
+/// Similar to [`alloc::rc::Weak`].
 ///
 /// Can be constructed with [`VRc::downgrade`] and use [`VWeak::upgrade`]
 /// to re-create the original VRc.
@@ -315,3 +325,115 @@ impl<VTable: VTableMetaDropInPlace + 'static, X: HasStaticVTable<VTable> + 'stat
         unsafe { core::mem::transmute(self) }
     }
 }
+
+/// Safety: The data VRc manages is held by `VRcInner`, which maintains its address when the VRc
+/// is moved.
+unsafe impl<VTable: VTableMetaDropInPlace + 'static, X> stable_deref_trait::StableDeref
+    for VRc<VTable, X>
+{
+}
+
+/// Safety: The data VRc manages is held by `VRcInner`, and a clone of a VRc merely clones the pointer
+/// *to* the `VRcInner`.
+unsafe impl<VTable: VTableMetaDropInPlace + 'static, X> stable_deref_trait::CloneStableDeref
+    for VRc<VTable, X>
+{
+}
+
+/// VRcMapped allows bundling a VRc of a type along with a reference to an object that's
+/// reachable through the data the VRc owns and that satisfies the requirements of a Pin.
+/// VRCMapped is constructed using [`VRc::map`] and, like VRc, has a weak counterpart, [`VWeakMapped`].
+pub struct VRcMapped<VTable: VTableMetaDropInPlace + 'static, MappedType: ?Sized> {
+    parent_strong: VRc<VTable, Dyn>,
+    object: *const MappedType,
+}
+
+impl<VTable: VTableMetaDropInPlace + 'static, MappedType: ?Sized> Clone
+    for VRcMapped<VTable, MappedType>
+{
+    fn clone(&self) -> Self {
+        Self { parent_strong: self.parent_strong.clone(), object: self.object }
+    }
+}
+
+impl<VTable: VTableMetaDropInPlace + 'static, MappedType: ?Sized> VRcMapped<VTable, MappedType> {
+    /// Returns a new [`VWeakMapped`] that points to this instance and can be upgraded back to
+    /// a [`Self`] as long as a `VRc`/`VMapped` exists.
+    pub fn downgrade(this: &Self) -> VWeakMapped<VTable, MappedType> {
+        VWeakMapped { parent_weak: VRc::downgrade(&this.parent_strong), object: this.object }
+    }
+
+    /// Create a Pinned reference to the mapped type.
+    ///
+    /// This is safe because the map function returns a pinned reference.
+    pub fn as_pin_ref(&self) -> Pin<&MappedType> {
+        unsafe { Pin::new_unchecked(&*self) }
+    }
+
+    /// This function allows safely holding a reference to a field inside the `VRcMapped`. In order to accomplish
+    /// that, you need to provide a mapping function `map_fn` in which you need to provide and return a
+    /// pinned reference to the object you would like to map. The returned `VRcMapped` allows obtaining
+    /// that pinned reference again using [`VRcMapped::as_pin_ref`].
+    ///
+    /// See also [`VRc::map`]
+    pub fn map<ReMappedType: ?Sized>(
+        this: Self,
+        map_fn: impl for<'r> FnOnce(Pin<&'r MappedType>) -> Pin<&'r ReMappedType>,
+    ) -> VRcMapped<VTable, ReMappedType> {
+        VRcMapped {
+            parent_strong: this.parent_strong.clone(),
+            object: map_fn(this.as_pin_ref()).get_ref(),
+        }
+    }
+
+    /// Returns a strong reference to the object that the mapping originates
+    /// from.
+    pub fn origin(this: &Self) -> VRc<VTable> {
+        this.parent_strong.clone()
+    }
+}
+
+impl<VTable: VTableMetaDropInPlace + 'static, MappedType: ?Sized> Deref
+    for VRcMapped<VTable, MappedType>
+{
+    type Target = MappedType;
+    fn deref(&self) -> &Self::Target {
+        // Safety: self.object was mapped from self.parent_strong, which the VRc
+        // keeps alive *and* pinned.
+        unsafe { &*self.object }
+    }
+}
+
+/// VWeakMapped allows bundling a VWeak with a reference to an object that's reachable
+/// from the object a successfully upgraded VWeak points to. VWeakMapped's API consists
+/// only of the ability to create clones and to attempt upgrading back to a [`VRcMapped`].
+pub struct VWeakMapped<VTable: VTableMetaDropInPlace + 'static, MappedType: ?Sized> {
+    parent_weak: VWeak<VTable, Dyn>,
+    object: *const MappedType,
+}
+
+impl<VTable: VTableMetaDropInPlace + 'static, MappedType: ?Sized> VWeakMapped<VTable, MappedType> {
+    /// Returns a new `VRcMapped` if some other instance still holds a strong reference to the owned
+    /// object. Otherwise, returns None.
+    pub fn upgrade(&self) -> Option<VRcMapped<VTable, MappedType>> {
+        self.parent_weak
+            .upgrade()
+            .map(|parent| VRcMapped { parent_strong: parent, object: self.object })
+    }
+}
+
+impl<VTable: VTableMetaDropInPlace + 'static, MappedType: ?Sized> Clone
+    for VWeakMapped<VTable, MappedType>
+{
+    fn clone(&self) -> Self {
+        Self { parent_weak: self.parent_weak.clone(), object: self.object }
+    }
+}
+
+impl<VTable: VTableMetaDropInPlace + 'static, MappedType> Default
+    for VWeakMapped<VTable, MappedType>
+{
+    fn default() -> Self {
+        Self { parent_weak: VWeak::default(), object: core::ptr::null() }
+    }
+}
