@@ -1,12 +1,5 @@
-/* LICENSE BEGIN
-    This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
-
-    SPDX-License-Identifier: GPL-3.0-only
-    This file is also available under commercial licensing terms.
-    Please contact info@sixtyfps.io for more information.
-LICENSE END */
+// Copyright © SixtyFPS GmbH <info@sixtyfps.io>
+// SPDX-License-Identifier: (GPL-3.0-only OR LicenseRef-SixtyFPS-commercial)
 
 #![allow(improper_ctypes_definitions)]
 
@@ -21,6 +14,7 @@ struct FooVTable {
 
 #[derive(Debug, const_field_offset::FieldOffsets)]
 #[repr(C)]
+#[pin]
 struct SomeStruct {
     e: u8,
     x: String,
@@ -151,3 +145,112 @@ fn rc_test_threading() {
     drop(weak);
     h.join().unwrap();
 }
+
+#[vtable]
+struct AppVTable {
+    drop_in_place: fn(VRefMut<AppVTable>) -> Layout,
+    dealloc: fn(&AppVTable, ptr: *mut u8, layout: Layout),
+}
+
+#[derive(Debug, const_field_offset::FieldOffsets)]
+#[repr(C)]
+#[pin]
+struct AppStruct {
+    some: SomeStruct,
+    another_struct: SomeStruct,
+}
+
+impl AppStruct {
+    fn new() -> VRc<AppVTable, Self> {
+        let string = Rc::new("hello".to_string());
+        let self_ = Self {
+            some: SomeStruct { e: 55, x: "_".into(), foo: string.clone() },
+            another_struct: SomeStruct { e: 100, x: "_".into(), foo: string.clone() },
+        };
+        VRc::new(self_)
+    }
+}
+
+impl App for AppStruct {}
+
+AppVTable_static!(static APP_STRUCT_TYPE for AppStruct);
+
+#[test]
+fn rc_map_test() {
+    fn get_struct_value(instance: &VRcMapped<AppVTable, SomeStruct>) -> u8 {
+        let field_ref = SomeStruct::FIELD_OFFSETS.e.apply_pin(instance.as_pin_ref());
+        *field_ref
+    }
+
+    let app_rc = AppStruct::new();
+
+    let some_struct_ref =
+        VRc::map(app_rc.clone(), |app| AppStruct::FIELD_OFFSETS.some.apply_pin(app));
+
+    // check clone() compiles
+    {
+        let _some_clone = some_struct_ref.clone();
+    }
+
+    let other_struct_ref =
+        VRc::map(app_rc.clone(), |app| AppStruct::FIELD_OFFSETS.another_struct.apply_pin(app));
+
+    let weak_struct_ref = VRcMapped::downgrade(&some_struct_ref);
+
+    {
+        let _weak_clone = weak_struct_ref.clone();
+    }
+
+    {
+        let strong_struct_ref = weak_struct_ref.upgrade().unwrap();
+        assert_eq!(get_struct_value(&strong_struct_ref), 55);
+    }
+
+    {
+        assert_eq!(get_struct_value(&other_struct_ref), 100);
+    }
+
+    drop(app_rc);
+
+    {
+        let strong_struct_ref = weak_struct_ref.upgrade().unwrap();
+        let e_field = SomeStruct::FIELD_OFFSETS.e.apply_pin(strong_struct_ref.as_pin_ref());
+        assert_eq!(*e_field, 55);
+    }
+
+    let double_map =
+        VRcMapped::map(other_struct_ref, |some| SomeStruct::FIELD_OFFSETS.e.apply_pin(some));
+    let double_map_weak = VRcMapped::downgrade(&double_map);
+    drop(double_map);
+    assert_eq!(*double_map_weak.upgrade().unwrap(), 100);
+
+    drop(some_struct_ref);
+
+    assert!(weak_struct_ref.upgrade().is_none());
+    assert!(double_map_weak.upgrade().is_none());
+
+    let def = VWeakMapped::<AppVTable, SomeStruct>::default();
+    assert!(def.upgrade().is_none());
+}
+
+#[test]
+fn rc_map_origin() {
+    let app_rc = AppStruct::new();
+
+    let some_struct_ref =
+        VRc::map(app_rc.clone(), |app| AppStruct::FIELD_OFFSETS.some.apply_pin(app));
+
+    drop(app_rc);
+
+    let strong_origin = VRcMapped::origin(&some_struct_ref);
+
+    drop(some_struct_ref);
+
+    let weak_origin = VRc::downgrade(&strong_origin);
+
+    assert_eq!(VRc::strong_count(&strong_origin), 1);
+
+    drop(strong_origin);
+
+    assert!(weak_origin.upgrade().is_none());
+}
